angular.module('sdbgui')
    .component('queryconfigs', {
        templateUrl: '/static/components/sdbgui-queryconfigs/sdbgui-queryconfigs.template.html',
        controller: ['$uibModal', 'sortingService', 'slashDB', function DbConfigController($uibModal, sortingService, slashdb) {
            var ctrl = this;
            ctrl.sort = sortingService;

            this.init = function () {
                this.updateConfigList();
            };

            this.updateConfigList = function () {
                slashdb.get('/querydef.json', {
                        params: {
                            guidata: ''
                        }
                    }, false, false)
                    .then(function (response) {
                        var configs = [];

                        ctrl.configsDict = response.data.querydefs;

                        for (var configId in ctrl.configsDict) {
                            configs.push(ctrl.configsDict[configId]);
                        }

                        ctrl.createNew = response.data.create;
                        ctrl.listAlert = '';
                        ctrl.configs = configs;
                    })
                    .catch(function (response) {
                        ctrl.listAlert = response.headers('Warning');
                        ctrl.configs = [];
                    });
            };

            this.openModal = function (configId) {
                var modalInstance = $uibModal.open({
                    templateUrl: '/static/components/sdbgui-queryconfigs/sdbgui-queryconfigs-modal.template.html',
                    size: 'lg',
                    controller: 'QueryConfigModalCtrl',
                    resolve: {
                        configId: function () {
                            return configId;
                        }
                    }
                });
                modalInstance.result
                    .catch(function () {
                        ctrl.updateConfigList();
                    }); // refresh config list after closing modal
            };

            this.executeQuery = function (queryId) {
                var query = ctrl.configsDict[queryId],
                    paramsDict = {};
                for (var i = 0, pl = query.parameters.length; i < pl; i++) {
                    var paramName = query.parameters[i];
                    paramsDict[paramName] = '';
                }
                query.paramsDict = paramsDict;
                ctrl.selectedQuery = query;
            };

            this.init();
        }]
    })
    .controller('QueryConfigModalCtrl', ['$timeout', '$scope', '$uibModalInstance', 'slashDB', 'configId',
        function ($timeout, $scope, $uibModalInstance, slashdb, configId) {
            var ctrl = this;

            ctrl.init = function () {
                $scope.suggestedMethods = $scope.getSuggestedMethods();
                $scope.newConfig = !Boolean(configId);
                $scope.configId = configId;
                $scope.okVisible = false;
                $scope.modalAlert = '';
                $scope.databases = []; // stores list of databases accessible by the user
                $scope.configData = {
                    editable: true
                }; // stores database definition model, some initial values
                $scope.extraConfigData = {};
                ctrl.updateConfig();
            };

            ctrl.updateConfig = function () {
                slashdb.get('/db.json', {
                        params: {
                            guidata: ''
                        }
                    }, false, false)
                    .then(function (response) {
                        $scope.databases = response.data;
                    });

                if (!$scope.newConfig) {
                    $scope.modalAlert = '';
                    slashdb.get('/querydef/' + $scope.configId + '.json', {
                            params: {
                                guidata: ''
                            }
                        }, false, false)
                        .then(function (response) {
                            $scope.configData = response.data;
                            $scope.suggestedMethods = $scope.getSuggestedMethods();
                        })
                        .catch(function (response) {
                            if (response.status == 403) $scope.clean(true);
                            $scope.modalAlert = response.headers('Warning');
                        });
                }

                // refresh slashDB.settings
                slashdb.setupSettings();
            };

            $scope.clean = function (andDisable) {
                $scope.configData = {
                    editable: !andDisable
                };
                $scope.extraConfigData = {};

            };

            $scope.delete = function () {
                $scope.modalAlert = '';
                slashdb.deleteQueryDef($scope.configId)
                    .then(function (response) {
                        $uibModalInstance.dismiss();

                        // refresh slashDB.settings
                        slashdb.setupSettings();
                    })
                    .catch(function (response) {
                        $scope.modalAlert = response.headers('Warning');
                    });
            };

            $scope.save = function () {
                $scope.modalAlert = '';
                if ($scope.newConfig) {
                    slashdb.createQueryDef($scope.configData)
                        .then(function (response) {
                            ctrl.blinkOK();
                            $scope.configId = $scope.configData.query_id;
                            $scope.newConfig = false;
                            ctrl.updateConfig();
                        })
                        .catch(function (response) {
                            $scope.modalAlert = response.headers('Warning');
                        });
                } else {
                    slashdb.updateQueryDef($scope.configId, $scope.configData)
                        .then(function (response) {
                            ctrl.blinkOK();
                            $scope.configId = $scope.configData.query_id;
                            ctrl.updateConfig();
                        })
                        .catch(function (response) {
                            $scope.modalAlert = response.headers('Warning');
                        });
                }
            };

            $scope.close = function () {
                $uibModalInstance.dismiss();
            };

            this.blinkOK = function () {
                $scope.okVisible = true;
                $timeout(function () {
                    $scope.okVisible = false;
                }, 1000);
            };

            var methodsMapping = {
                'select': {
                    re: /\bselect\b/i,
                    method: 'GET'
                },
                'insertOrExec': {
                    re: /\binsert|exec\b/i,
                    method: 'POST'
                },
                'update': {
                    re: /\bupdate\b/i,
                    method: 'PUT'
                },
                'delete': {
                    re: /\bdelete\b/i,
                    method: 'DELETE'
                }
            };

            var methodsMappingKeys = Object.keys(methodsMapping);
            var methodsMappingKeysL = methodsMappingKeys.length;

            $scope.getSuggestedMethods = function () {
                var suggestedMethods = {
                    'GET': false,
                    'POST': false,
                    'PUT': false,
                    'DELETE': false
                };

                if ($scope.configData == null) {
                    return suggestedMethods;
                }

                var sqlstr = $scope.configData.sqlstr,
                    searchTerm;

                for (var i = 0; i < methodsMappingKeysL; i++) {
                    searchTerm = methodsMapping[methodsMappingKeys[i]];
                    suggestedMethods[searchTerm.method] = searchTerm.re.exec(sqlstr) != null;
                }
                return suggestedMethods;
            };

            this.init();
        }
    ]);